/*
 * TelPac-Node/LinuX developed by Hans-J. Barthen, DL5DI based on tcp_call
 * from ax25-tools 0.0.8, enhanced for full-binary transfer and automatic
 * password protected user- and node-logon at Winlink 2000 servers.
 *
 * This code is like the original tools term of the GNU licence.
 *
 * In case of problems please contact dl5di@darc.de/dl5di@db0lj.ampr.org
 * Hans-Juergen Barthen, Franz-Josef-Str. 20, D-56642 Kruft
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <netdb.h>
#include <time.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>

#include "../user_io.h"


void alarm_handler(int sig)
{
}

int main(int argc, char **argv)
{
	char buffer[256];
	char inputline[256];
	char nodecall[20];
	char passwd[30];
	char hostname[30];
	int s, addrlen = sizeof(struct sockaddr_in);
	struct sockaddr_in addr;
	struct hostent *hp;
	struct servent *sp;
	struct ust userstat;
	time_t tstart;
	time_t tstop;
	int status = 0;
	char *cp;
	char *hhost;
	char *hport;
	char *hpass;
	FILE *config_file;
	FILE *host_file;

	paclen = 256;
	verbose = 1;

		
	while ((s = getopt(argc, argv, "o:p:q")) != -1) {
	    switch (s) {
		case 'o':
		case 'p':
			paclen = atoi(optarg);
			break;
		case 'q':
			verbose = 0;
			break;
		case ':':
		case '?':
			err("TelPac ACI: ERROR - invalid option usage\r\n");
			return 1;
	    }
	}

	if(verbose){
	    printf("\rTelPac-Node/LinuX %s (dl5di %s)\r\r",REV,__DATE__);
	}

	if (paclen < 1) {
	    err("TelPac ACI: ERROR - invalid paclen\r\n");
	}

	/*
	 * Open the socket into the kernel.
	 */
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	    sprintf(buffer, "TelPac ACI: ERROR - can't open socket: %s\r\n", strerror(errno));
	    err(buffer); 
	}

/*
 * read hostfile (telpac.hst), format "hostname:port:password"
 * try to connect host 
 * loop to other entries if connection fails
 */

	if((host_file = fopen(hostfile,"r")) != NULL) {

	    while(fgets(inputline, 80, host_file) != NULL) {
		cp = inputline;

		/* Jump over comments */
		if((*cp == '#') || (*cp == ';')) {
		    continue;
		}
		
		hhost = inputline;
		hport = NULL;
		hpass = NULL;

		/* search for start of 2nd token / port */
		while((*cp != '\0') && (*cp != '\r') && (*cp != '\n')) {
		    if(*cp == ':') {
			*cp = '\0';
			cp++;
			hport = cp;
			break;
		    }
		    cp++;
		}
		if (strlen(hhost) == 0 ) {
		    err("TelPac ACI: ERROR - missing hostname in configfile\r\n");
		}

		/*
		 * Resolve the hostname
		 */
		hp = gethostbyname(hhost);
		if (hp == NULL) {
		    sprintf(buffer,"TelPac ACI: ERROR - Unknown host '%s'. Try next line.\r\n", hhost);
		    msg(buffer);
		    status++;
		    continue;
		}
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;


		/* search for start of 3rd token / password */
		while((*cp != '\0') && (*cp != '\r') && (*cp != '\n')){
		    if(*cp == ':') {
			*cp = '\0';
			cp++;
			hpass = cp;
			break;
		    }
		    cp++;
		}

		if (strlen(hport) == 0 ) {
		    sprintf(buffer,"TelPac ACI: ERROR - missing service '%s'. Try next line.\r\n", hport);
		    msg(buffer);
		    status++;
		    continue;
		}

		/*
		 * Resolve the service name.
		 */

		if ((sp = getservbyname(hport, "tcp")) != NULL) {
		    addr.sin_port = sp->s_port;
		} else {
		    addr.sin_port = htons(atoi(hport));
		}
		
		if (addr.sin_port == 0) {	/* you should never get here ! */
		    sprintf(buffer,"TelPac ACI: ERROR - unknown service '%s' ?\r\n", hport);
		    err(buffer);
		}

		/*
		 * Now the rest must be a password.
		 */

		while(*cp != '\0'){
		    if((*cp == '\r') || (*cp == '\n')){
			*cp = '\0';
			break;
		    }
		    cp++;
		}

		if (strlen(hpass) == 0 ) {
		    sprintf(buffer,"TelPac ACI: ERROR - missing password at host '%s'. Try next host.\r\n",hhost);
		    msg(buffer);
		    status++;
		    continue;
		}

		strcpy(passwd,hpass);

	       /*
	        * Connect to Winlink2000 server
		*/

		strcpy(hostname,hp->h_name);

		if (verbose) {
		    printf("\r\n*** Connecting to %s ...\r\n", hostname);
		}


	       /*
		* If no response in 30 seconds, go away.
		*/
		alarm(30);

		signal(SIGALRM, alarm_handler);

	       /*
		* Lets try and connect to the far end.
		*/
		if (connect(s, (struct sockaddr *)&addr, addrlen) != 0) {
		    sprintf(buffer, "can't connect %s (%s), try next.\r", hostname, strerror(errno));
		    msg(buffer);
		    status++;
		} else {
		    status = 0;
		}
		
	       /*
		* We got there.
		*/
		alarm(0);

		if(status == 0) {
		    break;
		}

	    }	/* end of loop through hostfile */

	} else {
	    err("TelPac ACI: ERROR ! Missing Host-File !\r\n");
	}

	fclose(host_file);

	if(status > 0) {
	    sprintf(buffer, "TelPac ACI: ERROR ! No Host-Connection ! Tried %d hosts.\r", status);
	    err(buffer);
	}

	/*
	 * here the server connection is established !
	 */

	time(&tstart);

	/*
	 * Read configfile, start with nodecall followed by ctext
	 */

	if((config_file = fopen(configfile,"r")) != NULL) {

	    if(fgets(nodecall, 20, config_file) != NULL) {

		if (strlen(nodecall) > 10) {
		    err("TelPac ACI: ERROR ! Invalid Node-Callsign !\r");
		} else {
		    cp = nodecall;
		    while(*cp != '\0') {
			*cp = toupper(*cp);
			if ((*cp == '\n') || (*cp == '\r')){
			    *cp = '\0';
			    break;
			}
			cp++;
		    }
		    if (verbose) {
			printf("*** Nodecall: >%s<\r\n",nodecall);
		    }
		}
	    } else {
		err("TelPac ACI: ERROR IN CONFIG FILE - Missing Node-Callsign !\r\n");
	    }

	    fclose(config_file);

	} else {
	    err("TelPac ACI: ERROR ! Missing Config-File !\r\n");
	}

	if (verbose) {
		printf("*** Connected\r\n");
	}

	userstat = aci(s, passwd, nodecall, hostname);

	time(&tstop);

	log_aci(hostname,userstat,tstart,tstop);

	if (verbose) {
	    printf("\r\n*** Disconnected\r\n");
	}
	return 0;
}
