/*
 * TelPac-Node/LinuX developed by Hans-J. Barthen, DL5DI based on tcp_call
 * from ax25-tools 0.0.8, enhanced for full-binary transfer and automatic
 * password protected user- and node-logon at Winlink 2000 servers.
 *
 * This code is like the original tools term of the GNU licence.
 *
 * In case of problems please contact dl5di@darc.de/dl5di@db0lj.ampr.org
 * Hans-Juergen Barthen, Franz-Josef-Str. 20, D-56642 Kruft
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>

#include "../user_io.h"
#include "../tpcfg.h"

#define BUFLEN	8192

int paclen = 128;  // set to "safe" default value

/* This is for select_loop() */
static unsigned char buf[BUFLEN];

#define min(a,b)	((a) < (b) ? (a) : (b))

void err(char *message)
{
    FILE *err_file;
    FILE *log_file;
    time_t tstop;
	write(STDOUT_FILENO, message, strlen(message));
	time(&tstop);

	if((err_file = fopen(errfile, "a")) != NULL) {
	    fprintf(err_file, "Logout '%s' %-24.24s ERROR !\n",message, ctime(&tstop));
	    fclose(err_file);
	}

	if((log_file = fopen(logfile, "a")) != NULL) {
	    fprintf(log_file, "Logout %-24.24s ERROR !\n",ctime(&tstop));
	    fclose(log_file);
	}

	exit(1);
}


void msg(char *message)
{
    FILE *err_file;
    time_t tstop;

	if((err_file = fopen(errfile, "a")) != NULL) {
	    time(&tstop);
	    fprintf(err_file, "%s %s", message, ctime(&tstop));
	    fclose(err_file);
	}

	return;
}


struct ust aci(int s, char *passwd, char *nodecall, char *hostname)
{
	fd_set read_fd;
	int m;
	int n = 0;
	int loop = 0;
	int st = 0;
	unsigned char *cp;
	struct ust rc;
        char mybuf[80];
        int res;

	rc.errcode = 0;
	rc.bytes_recv = 0;
	rc.bytes_sent = 0;
	
	if (fcntl(s, F_SETFL, O_NONBLOCK) < 0) {
	        // probably should change this to preserve any
	        // other flags that may be set  E.G.
	        //   arg = fcntl(mysock, F_GETFL, NULL); 
  		//   arg |= O_NONBLOCK; 
		//    fcntl(mysock, F_SETFL, arg);   
		close(s);
		rc.errcode = -1;	/* can not open socket */
		return(rc);
	}

	/*
	 * Loop until one end of the connection goes away.
	 */
	for (;;) {
		FD_ZERO(&read_fd);
		FD_SET(s, &read_fd);

		res = select(s + 1, &read_fd, NULL, NULL, NULL);

                //sprintf(mybuf,"Info: Res = %d, s = %d\r\n", res, s);
                //send(STDOUT_FILENO, mybuf, strlen(mybuf), 0);
                
                if (FD_ISSET(s, &read_fd))
                // If the socket indicated ready to read but returns 0 bytes
                // on a receive then the other end has closed.
                // Only ask for one byte to minimize work done by the following
                // recv statement as it is called every time through the loop.
                  {
                  //n = recv(s, buf, 1, MSG_PEEK | MSG_DONTWAIT);
                  if ((n = recv(s, buf, 1, MSG_PEEK | MSG_DONTWAIT)) == 0)
                    {
                    sprintf(mybuf,"INFO: Connection closed by PMBO\r\n");
                    send(STDOUT_FILENO, mybuf, strlen(mybuf), 0);                                
                    rc.errcode = 0;
                    sleep(1);
                    close(s);
                    return(rc);
                    }
                    
                  } //if (FD_ISSET(s, &read_fd))

		
		select(s + 1, &read_fd, NULL, NULL, NULL);

		if (FD_ISSET(s, &read_fd)) {
			m = 0;
			while ((n = read(s, buf, BUFLEN)) > 0){
		    	rc.bytes_recv += n;
		    	m = n;
	    	    	if(verbose){
		        	write(STDOUT_FILENO, buf, n);
		    	}

		    	if (n < 0 && errno != EAGAIN) {
				rc.errcode = -1;
				break;
		    	}
		}
	

		switch (st) {

		    case 0:
			    /* get login prompt */
			    if(*(buf+m-2) == ':') {
				st++;
			    }

	    		    if(verbose) {
				n = sprintf(buf,"*** 0/%d:%d >%c<\r\n",st,m,*(buf+m-2));
			        write(STDOUT_FILENO, buf, n);
	    		    }

			    /*
			     * send node logon to server after recept of 1st buffer
			     */

	    		    n = sprintf(buf,".%s\r%s\rTelPac %s\r",nodecall,passwd,nodecall);
	    		    cp = buf;

	    		    if(verbose) {
				write(STDOUT_FILENO, buf, n);
	    		    }

			    write(s, buf, n);
	    		    rc.bytes_sent += n;
		
			    if (n < 0 && errno != EAGAIN) {
				rc.errcode = -3;
			    }

			    break;

		    case 1:
			    /*
			     * - get telnet logon text and welcome message
			     * - wait for '>' at end of prompt
			     */
			     
			    if(m > 0){

				st++;
	    			if(verbose) {
				    n = sprintf(buf,"*** 1/%d:%d >%c<\r\n",st,m,*(buf+m-2));
			    	    write(STDOUT_FILENO, buf, n);
	    			}

				/*
			         *	send BYE-command
			         */

				n = sprintf(buf,"BYE\r\n");
				cp = buf;

				if(verbose){
			    	    write(STDOUT_FILENO, buf, n);
				}

				write(s, buf, n);
				rc.bytes_sent += n;

				if (n < 0 && errno != EAGAIN) {
				    rc.errcode = -4;
				}
				loop = TIMEOUT - 5;
			    }
			    break;

		    case 2:
			    /*
			     *	get logoff message
			     */

			    if(m > 1){
				st++;
	    			if(verbose) {
				    n = sprintf(buf,"*** 2/%d:%d\r\n",st,m);
			    	    write(STDOUT_FILENO, buf, n);
	    			}
			    }
			    break;

		    case 3:
	    		    if(verbose) {
				n = sprintf(buf,"*** END\r\n");
			        write(STDOUT_FILENO, buf, n);
	    		    }
			    rc.errcode = 0;
			    return(rc);

		    default:
			    break;

		}	/* end switch */

		loop++;

		if(loop > TIMEOUT) {

	    	    if(verbose) {
			n = sprintf(buf,"*** TIMEOUT\r\n");
		        write(STDOUT_FILENO, buf, n);
	    	    }
		    rc.errcode = 1;
		    close(s);
		    return(rc);
		}

		if (rc.errcode < 0) {
		    if(verbose){
    			n = sprintf(buf,"TX-Error! (%d)\r\n", rc.errcode);
			write(STDOUT_FILENO, buf, n);
		    }
		    close(s);
		    return(rc);
		}
		
		sleep(1);

	    } else {	/* FD_ISSET */
	    
	    	if(verbose) {
		    n = sprintf(buf,"*** Server closed\r\n");
		    write(STDOUT_FILENO, buf, n);
	    	}
		rc.errcode = 0;
		close(s);
		return(rc);
	    }

	}	/* end FOR */

	return(rc);
}


/*
 *  write logentry for automatic checkin sessions
 */


int log_aci(char* hostname, struct ust userstat, time_t tstart, time_t tstop)
{
    double tused;
    float rate;
    FILE *log_file;

    if((log_file = fopen(logfile, "a")) != NULL) {
	time(&tstop);
	tused = difftime(tstop, tstart);
        rate = (userstat.bytes_sent + userstat.bytes_recv) / tused;
	fprintf(log_file, "ACI Host         %-24.24s %s tx:%li rx:%li %.1fs %.1f Bytes/s (%d)\n",ctime(&tstop),hostname,userstat.bytes_sent,userstat.bytes_recv,tused,rate,userstat.errcode);
	fclose(log_file);
    }
    return (0);
}
